import 'package:flutter/material.dart';
import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:petitbac/common/cubit/nav/nav_cubit_pages.dart';

import 'package:petitbac/cubit/activity/activity_cubit.dart';

class ApplicationConfig {
  static const String parameterCodeItemsCount = 'activity.itemsCount';
  static const String parameterCodeTimerValue = 'activity.timerValue';

  static const String itemsCountValueNoLimit = 'nolimit';
  static const String itemsCountValueShort = 'short';
  static const String itemsCountValueMedium = 'medium';
  static const String itemsCountValueLong = 'long';

  static const String timerValueNoTimer = 'nolimit';
  static const String timerValueLow = 'low';
  static const String timerValueMedium = 'medium';
  static const String timerValueHigh = 'high';

  static final ApplicationConfigDefinition config = ApplicationConfigDefinition(
    appTitle: 'Petit Bac',
    activitySettings: [
      // items count
      ApplicationSettingsParameter(
        code: parameterCodeItemsCount,
        values: [
          ApplicationSettingsParameterItemValue(
            value: itemsCountValueNoLimit,
            color: Colors.grey,
            text: '⭐',
          ),
          ApplicationSettingsParameterItemValue(
            value: itemsCountValueShort,
            color: Colors.green,
            text: '💬 5',
          ),
          ApplicationSettingsParameterItemValue(
            value: itemsCountValueMedium,
            color: Colors.orange,
            text: '💬 10',
            isDefault: true,
          ),
          ApplicationSettingsParameterItemValue(
            value: itemsCountValueLong,
            color: Colors.red,
            text: '💬 20',
          ),
        ],
        intValueGetter: (value) {
          const Map<String, int> intValues = {
            ApplicationConfig.itemsCountValueNoLimit: 0,
            ApplicationConfig.itemsCountValueShort: 5,
            ApplicationConfig.itemsCountValueMedium: 10,
            ApplicationConfig.itemsCountValueLong: 20,
          };
          return intValues[parameterCodeItemsCount] ?? 0;
        },
      ),

      // timer value
      ApplicationSettingsParameter(
        code: parameterCodeTimerValue,
        values: [
          ApplicationSettingsParameterItemValue(
            value: timerValueNoTimer,
            color: Colors.grey,
            text: '⭐',
          ),
          ApplicationSettingsParameterItemValue(
            value: timerValueLow,
            color: Colors.green,
            text: '⏲️ 5',
          ),
          ApplicationSettingsParameterItemValue(
            value: timerValueMedium,
            color: Colors.orange,
            text: '⏲️ 30',
            isDefault: true,
          ),
          ApplicationSettingsParameterItemValue(
            value: timerValueHigh,
            color: Colors.red,
            text: '⏲️ 90',
          ),
        ],
        intValueGetter: (value) {
          const Map<String, int> intValues = {
            timerValueNoTimer: 0,
            timerValueLow: 5,
            timerValueMedium: 30,
            timerValueHigh: 90,
          };
          return intValues[parameterCodeTimerValue] ?? 0;
        },
      ),
    ],
    startNewActivity: (BuildContext context) {
      BlocProvider.of<ActivityCubit>(context).startNewActivity(context);
      BlocProvider.of<NavCubitPage>(context).goToPageGame();
    },
    deleteCurrentActivity: (BuildContext context) {
      BlocProvider.of<ActivityCubit>(context).deleteSavedActivity();
    },
    resumeActivity: (BuildContext context) {
      BlocProvider.of<ActivityCubit>(context).resumeSavedActivity();
      BlocProvider.of<NavCubitPage>(context).goToPageGame();
    },
  );
}
